
'''
ZhuWJ
'''
# defin return code


class RADAR_COMMON:
    def __init__(self) -> None:
     
        self.RETURN_CODE_SUCESSED = 1
        self.RETURN_CODE_PARAM_ERROR = -1
        self.RETURN_CODE_OPEN_ERROR = -2
        self.RETURN_CODE_FORMAT_ERROR = -3
        self.RETURN_CODE_DATA_ERROR = -4




#! /usr/bin/python3
#
def pgmb_write ( file_name,params, width, height, maxval, gray ):

#*****************************************************************************80
#
## PGMB_WRITE writes a binary PGM graphics file.
#
#  Licensing:
#
#    This code is distributed under the GNU LGPL license. 
#
#  Modified:
#
#    14 September 2018
#
#  Author:
#
#    John Burkardt
#
#  Parameters:
#
#    Input, string FILE_NAME, the name of the file.
#
#    Input, integer WIDTH, HEIGHT, the width and height of the graphics image.
#
#    Input, integer MAXVAL, the maximum allowed gray value.
#
#    Input, integer GRAY[WIDTH*HEIGHT], values between 0 and MAXVAL.
#
  import numpy as np
  import struct

  file_handle = open ( file_name, 'wb' )
#
#  Set up the header.
# 
  # params['obstimestr'] = 201609281600
  # params['left_lon'] = 109
  # params['right_lon'] = 117
  # params['bottom_lat'] = 33
  # params['upper_lat'] = 37

  pgm_header = 'P5\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 

  pgm_header = '# composite_area FIN\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# obstime %s\n'%params['obstimestr']
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# producttype CAPPI\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# productname LOWEST\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# param CorrectedReflectivity\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# metersperpixel_x 999.674053\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# metersperpixel_y 999.62859\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# projection radar {\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# type stereographic\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# centrallongitude 110\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# centrallatitude 90\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# truelatitude 60\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# bottomleft %.3f %.3f\n'%(params['left_lon'],params['bottom_lat'])
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# topright %.3f %.3f\n'%(params['right_lon'],params['upper_lat'])
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = '# }\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  # b'# composite_area FIN\n'
  # b'# projection_name SUOMI1\n'

  # b'# obstime 201609281600\n'
  # b'# producttype CAPPI\n'
  # b'# productname LOWEST\n'
  # b'# param CorrectedReflectivity\n'
  # b'# metersperpixel_x 999.674053\n'
  # b'# metersperpixel_y 999.62859\n'
  # b'# projection radar {\n'
  # b'# type stereographic\n'
  # b'# centrallongitude 25\n'
  # b'# centrallatitude 90\n'
  # b'# truelatitude 60\n'
  # b'# bottomleft 18.600000 57.930000\n'
  # b'# topright 34.903000 69.005000\n'
  # b'# }\n'

  pgm_header = f'{width} {height}\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
  pgm_header = f'{maxval}\n'
  file_handle.write ( bytearray ( pgm_header, 'ascii' ) ) 
#
#  Convert 2D array to 1D vector.
#
  grayV = np.reshape ( gray, width * height )
#
#  Pack entries of vector into a string of bytes, replacing each integer
#  as an unsigned 1 byte character.
#
  grayB = struct.pack ( '%sB' % len(grayV), *grayV )
  file_handle.write ( grayB )
 
  file_handle.close ( )

  return

def pgmb_write_test ( ):

#*****************************************************************************80
#
## PGMB_WRITE_TEST tests PGMB_WRITE.
#
#  Licensing:
#
#    This code is distributed under the GNU LGPL license. 
#
#  Modified:
#
#    14 May 2017
#
#  Author:
#
#    John Burkardt
#
  import numpy as np
  import platform

  print ( '' )
  print ( 'PGMB_WRITE_TEST:' )
  print ( '  Python version: %s' % ( platform.python_version ( ) ) )
  print ( '  PGMB_WRITE writes a binary PGM graphics file.' )

  file_name = '/Users/wenjianzhu/Downloads/pgmb_io_test.pgm'
  width = 24
  height = 7
  maxval = 255

  gray = np.array \
  ( [ \
    [ 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0], \
    [ 0,  64,  64,  64,  64,   0,   0, 128, 128, 128, 128,   0,   0, 192, 192, 192, 192,   0,   0, 255, 255, 255, 255,   0], \
    [ 0,  64,   0,   0,   0,   0,   0, 128,   0,   0,   0,   0,   0, 192,   0,   0,   0,   0,   0, 255,   0,   0, 255,   0], \
    [ 0,  64,  64,  64,   0,   0,   0, 128, 128, 128,   0,   0,   0, 192, 192, 192,   0,   0,   0, 255, 255, 255, 255,   0], \
    [ 0,  64,   0,   0,   0,   0,   0, 128,   0,   0,   0,   0,   0, 192,   0,   0,   0,   0,   0, 255,   0,   0,   0,   0], \
    [ 0,  64,   0,   0,   0,   0,   0, 128, 128, 128, 128,   0,   0, 192, 192, 192, 192,   0,   0, 255,   0,   0,   0,   0], \
    [ 0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0]  \
  ] )

  pgmb_write ( file_name, width, height, maxval, gray )

  print ( '' )
  print ( '  Graphics data stored in file "%s".' % ( file_name ) )
#
#  Terminate.
#
  print ( '' )
  print ( 'PGMB_WRITE_TEST:' )
  print ( '  Normal end of execution.' )
  return

if ( __name__ == '__main__' ):
  pgmb_write_test ( )



# _*_ coding: utf-8 _*_

'''
将单站qpe产品进行拼图处理
'''
# %%
import os
import numpy as np
import glob
from datetime import  datetime,timedelta
from netCDF4 import Dataset
import xarray as xr
import pickle
import warnings
warnings.filterwarnings('ignore')
import configparser
#添加时间控制器
import time
import sched
import sys
from multiprocessing import Pool,cpu_count,freeze_support
import pandas as pd
schedule = sched.scheduler(time.time, time.sleep)

BDEBUG=False

# sub function for reading config file
def ConfigFetchError(BaseException):
    pass

def _get_config_from_rcfile(rcfile):
    """
    Get configure information from config_dk_met_io.ini file.
    """

    print(rcfile)
    try:
        config = configparser.ConfigParser()
        config.read(rcfile,encoding='UTF-8')
    except IOError as e:
        raise ConfigFetchError(str(e))
    except Exception as e:
        raise ConfigFetchError(str(e))

    return config

def get_datetime_from_filename1(filename):
    timstr = filename[15:29]
    fyear = int(timstr[0:4])
    fmonth = int(timstr[4:6])
    fday = int(timstr[6:8])
    fhour = int(timstr[8:10])
    fmin = int(timstr[10:12])
    fsec = int(timstr[12:14])
    ft = datetime(fyear,fmonth,fday,fhour,fmin,fsec).timestamp()
    return ft

# Z9973_20240903_160311_01hr.nc
def get_datetime_from_filename2(filename):
    timstr = filename[6:14] + filename[15:21]
    fyear = int(timstr[0:4])
    fmonth = int(timstr[4:6])
    fday = int(timstr[6:8])
    fhour = int(timstr[8:10])
    fmin = int(timstr[10:12])
    fsec = int(timstr[12:14])
    ft = datetime(fyear,fmonth,fday,fhour,fmin,fsec).timestamp()
    return ft



def do_single(curt:datetime):


    # if BDEBUG:
    #     curt = datetime(2021,5,8,8,20,0)
    # else:
    #     curt = datetime.utcnow()   

    try:
        if not os.path.exists(PATH_TMP):
            os.makedirs(PATH_TMP)

        for acc_hours in ACC_HOURS:
            print('当前累积时段 = %d 小时'%acc_hours)
            print(curt.strftime('当前执行时间为：%Y年%m月%d日%H时%M分(UTC)'))
            # acc_hours = 3
            # #降水产品格点设置
            # START_LAT= 25
            # END_LAT  = 30
            # START_LON= 109
            # END_LON  = 114
            # MOSAIC_RESO = 0.01 #网格分辨率，经纬度，度

            mosaic_lats = np.arange(START_LAT,END_LAT+MOSAIC_RESO/2,MOSAIC_RESO)
            mosaic_lons = np.arange(START_LON,END_LON+MOSAIC_RESO/2,MOSAIC_RESO)
            glons,glats = np.meshgrid(mosaic_lons,mosaic_lats)

            GRID_XNUM = len(mosaic_lons)
            GRID_YNUM = len(mosaic_lats)

            
            # 获取所有单站数据的时间戳
            allfiles = glob.glob(PATH_QPE + '/*/*')
            dic_radfile = {rad:[] for rad in RADARS}
            dic_radtime = {rad:[] for rad in RADARS}
            
            for rd in RADARS:
                curfilepath = PATH_QPE + os.sep + rd
                print(curfilepath)
                files = glob.glob(curfilepath + os.sep + '*%02dhr.nc'%acc_hours)
                files = [file[-29::] for file in files]
                files = sorted(files,reverse=True)
                dic_radfile[rd] = files
                for ff in files:
                    dic_radtime[rd].append(get_datetime_from_filename2(ff))

            # 把过去30分钟的数据，每间隔6分钟的都处理一遍
            cur_min = (curt.minute // MOSAIC_STEP)*MOSAIC_STEP
            vlid_t = datetime(curt.year,curt.month,curt.day,curt.hour,cur_min,0)
            checktimes = np.arange(vlid_t.timestamp(),vlid_t.timestamp()-REDO_MINS*60-1,-MOSAIC_STEP*60)

            mosaic_path = PATH_QPE_MOSAIC + os.sep + '%02dhr'%acc_hours
            if not os.path.exists(mosaic_path):
                try:
                    os.makedirs(mosaic_path)
                except:
                    print('创建目录失败：' + mosaic_path)

            for mn in checktimes:
                print(datetime.fromtimestamp(mn).strftime('正在执行 %Y年%m月%d日%H时%M分(UTC) 的拼图......'))
                oldlist=[]
                
                outname = 'QPE_MOSAIC_' + datetime.fromtimestamp(mn).strftime('%Y%m%d%H%M') + '_%02dhr.nc'%acc_hours
                # 先判断该拼图是否已经存在，并且里面的文件列表无更新，如果是，就不重复处理，直接跳过
                if os.path.exists(mosaic_path + os.sep + outname):
                    try:
                        tmplst = xr.open_dataset(mosaic_path + os.sep + outname)
                        oldlist = tmplst.basefile_info
                    except:
                        print('读取文件失败：' + mosaic_path + os.sep + outname)
                
                validfnum=0
                dic_validfile = {rad:[] for rad in RADARS}
                for rd in RADARS:
                    if len(dic_radtime[rd]) == 0:
                        # 删除该键值
                        dic_validfile.pop(rd)
                        # dic_validfile[rd] = []
                        # print(rd + ' 无数据！')
                        continue
                    else:
                        tmpt = list(abs(np.array(dic_radtime[rd]) - mn))
                        #只查找时间差在正负3分钟以内的文件
                        if min(tmpt) <= 180:
                            dic_validfile[rd] = dic_radfile[rd][tmpt.index(min(tmpt))]
                            print('valid file: ' + dic_validfile[rd])
                            validfnum +=1
                        else:
                            pass
                            # print(dic_radfile[rd])
                            # print('%s 最短时间差为：%.1f秒，超过了三分钟的匹配时限'%(rd,min(tmpt)))
                        
                #查到当前时间的所有雷达的匹配数据后，进行拼图处理
                if validfnum == 0:
                    print(datetime.fromtimestamp(mn).strftime('%Y年%m月%d日%H时%M分(UTC)') + ' 有效数据个数为0！')
                    continue
                #   
                mosaic_grid=np.zeros([len(dic_validfile),GRID_YNUM,GRID_XNUM],dtype=float)    
                # 依次读取单站qpe产品，然后拼图
                rflag=0
                # dic_fileinfo = {key:[] for key in dic_validfile.keys()}
                fileinfo_all = []
                bbflag=False
                for rd in dic_validfile.keys():
                    curfilepath = PATH_QPE + os.sep + rd
                    curfilename = dic_validfile[rd]
                    if len(curfilename) == 0:
                        continue
                    if not os.path.exists(curfilepath + os.sep + curfilename):
                        print(curfilepath + os.sep + curfilename + ' 文件不存在！')
                        continue
                    try:
                        # grid = pyart.io.read_grid(curfilepath + os.sep + curfilename)
                        grid = xr.open_dataset(curfilepath + os.sep + curfilename)
                        if not set(grid.basefile_info) <= set(oldlist):
                            bbflag = True
                            break
                    except:
                        print(curfilepath + os.sep + curfilename + ' 文件读取失败, continue！')
                        continue
                if not bbflag:
                    print(outname + ' 拼图文件所用原始文件无更新，不重复处理！')
                    continue

                used_sites=[]
                used_lons=[]
                used_lats=[]
                used_hours=[]

                for rd in dic_validfile.keys():
                    curfilepath = PATH_QPE + os.sep + rd
                    curfilename = dic_validfile[rd]
                    if len(curfilename) == 0:
                        continue
                    if not os.path.exists(curfilepath + os.sep + curfilename):
                        print(curfilepath + os.sep + curfilename + ' 文件不存在！')
                        continue
                    try:
                        # grid = pyart.io.read_grid(curfilepath + os.sep + curfilename)
                        grid = xr.open_dataset(curfilepath + os.sep + curfilename)
                    except:
                        print(curfilepath + os.sep + curfilename + ' 文件打开失败！')
                        continue
                    if set(grid.basefile_info) <= set(oldlist):
                        print('%s 站原始文件无变化: '%rd)

                    print('%s 站需制作新产品或原始文件有更新，处理：'%rd + curfilename + ' loaded!')

                    # check site location
                    # 先检查rd是否在ALL_RADARS的索引里面
                    if rd not in ALL_RADARS.index:
                        print('%s 站不在站号表文件radarsites_national.pkl中，请注意！'%rd)
                    
                    if abs(grid.radlon - ALL_RADARS.loc[rd]['经度']) > 0.1 or abs(grid.radlat - ALL_RADARS.loc[rd]['纬度']) > 0.1:
                        print('%s 站的经度或纬度有异常，基数据中的记录的位置和站点表中记录的不一致，误差超过了0.1度，请注意，这里暂时以数据中记录的为准！'%rd)
                        print('基数据中记录的经度为%.3f, 纬度为%.3f'%(grid.radlon,grid.radlat))
                        print('站号表文件radarsites_national.pkl中记录的经度为%.3f, 纬度为%.3f'%(ALL_RADARS.loc[rd]['经度'],ALL_RADARS.loc[rd]['纬度']))

                    used_sites.append(rd)
                    used_lons.append(grid.radlon)
                    used_lats.append(grid.radlat)

                    for tmpff in grid.basefile_info:
                        fileinfo_all.append(tmpff)
                    used_hours.append(grid.used_hours) 
                    #组合
                    mosaic_grid[rflag,:,:] = grid['qpe_mosaic'].values
                    
                    rflag += 1

                mosaic_grid = np.nanmean(mosaic_grid,axis=0)

                # 输出到nc格式
                try:
                    ncfile = Dataset(mosaic_path + os.sep + outname + '.lock', 'w', format='NETCDF4')
                except:
                    try:
                        os.remove(mosaic_path + os.sep + outname+ '.lock')
                        try:
                            ncfile = Dataset(mosaic_path + os.sep + outname+ '.lock', 'w', format='NETCDF4')
                        except:
                            print(mosaic_path + os.sep + outname +  '.lock' + ' 文件创建失败，删除成功，但无法再新建，跳过！')
                            continue
                    except:
                        print(mosaic_path + os.sep + outname + '.lock' + ' 文件创建失败，并且无法通过程序删除旧的同名文件，程序跳过！')
                        continue

                hreso = MOSAIC_RESO
                        
                # global
                ncfile.description = 'RADAR QPE MOSAIC '
                # ncfile.author = 'Zhu WenJian, zhuwj@cma.gov.cn'
                ncfile.reso = hreso
                ncfile.start_lat = START_LAT
                ncfile.end_lat = END_LAT
                ncfile.start_lon = START_LON
                ncfile.end_lon = END_LON
                ncfile.product_time = datetime.fromtimestamp(mn).strftime('%Y%m%d%H%M') 
                ncfile.last_update = curt.strftime('%Y%m%d%H%M') 
                ncfile.basefile_info = list(fileinfo_all)
                ncfile.used_sites = list(used_sites)
                ncfile.used_lons = list(used_lons)
                ncfile.used_lats = list(used_lats)
                ncfile.used_hours = used_hours
                
                unused_sites = [ss for ss in RADARS if not ss in used_sites]
                ncfile.unused_sites = list(unused_sites)
                unused_lons = [ALL_RADARS.loc[ll]['经度'] for ll in unused_sites]
                unused_lats = [ALL_RADARS.loc[ll]['纬度'] for ll in unused_sites]
                ncfile.unused_lons = list(unused_lons)
                ncfile.unused_lats = list(unused_lats)

                # dimensions
                nlons = mosaic_lons
                nlats = mosaic_lats

                ncfile.createDimension('longitude', len(nlons))
                ncfile.createDimension('latitude', len(nlats))

                # variables
                longitude = ncfile.createVariable('longitude', np.dtype('float32').char, ('longitude',))
                longitude.units = 'degrees_east'
                longitude.long_name = 'Longitude'
                longitude.CoordinateAxisType = 'Lon'

                latitude = ncfile.createVariable('latitude', np.dtype('float32').char, ('latitude',))
                latitude.units = 'degrees_north'
                latitude.long_name = 'Latitude'
                latitude.CoordinateAxisType = 'Lat'

                # H	unsigned short
                qpe_mosaic = ncfile.createVariable('qpe_mosaic', 'H', ('latitude', 'longitude'), zlib=True, least_significant_digit=1)
                qpe_mosaic.unit_scale = 0.1
                qpe_mosaic.long_name = "QPE MOSAIC"
                qpe_mosaic.units='mm'
                # data
                qpe_mosaic[:,:] = mosaic_grid * 10
                longitude[:] = nlons
                latitude[:] = nlats

                ncfile.close()
                try:
                    os.rename(mosaic_path + os.sep + outname + '.lock',mosaic_path + os.sep + outname)
                except:
                    print(mosaic_path + os.sep + outname + ' 重命名失败！')

                print(mosaic_path + os.sep + outname + ' done!')
    except :
        print(mosaic_path + os.sep + outname + ' + do_single 失败，跳过！')

def get_time_range_from_filelist():
    #从数据目录里面获取时间范围，用于后续处理
    trange ={}
    # 对雷达进行遍历
    for rd in RADARS:
        curpath = PATH_QPE + os.sep + rd
        if not os.path.exists(curpath):
            continue
        ff = os.listdir(curpath)
        ft=[]
        for f in ff:
            if f[-3:] == 'nc':
                continue
            #从文件名中获取时间
            t = get_datetime_from_filename2(f)
            ft.append(t)
        #对ft进行排序
        ft.sort()
        trange[rd] = [ft[0],ft[-1]]
        #计算时间范围
        trange[rd].append(int((trange[rd][1]-trange[rd][0])/3600))
        print('Radar %s has %d files, start time is %s, end time is %s, time range is %d hours'%(rd,len(ft),datetime.fromtimestamp(trange[rd][0]).strftime('%Y%m%d%H%M'),datetime.fromtimestamp(trange[rd][1]).strftime('%Y%m%d%H%M'),trange[rd][2]))

    #对所有雷达的时间范围进行合并，并将tmin和tmax存为datetime类型

    tmin = trange[RADARS[0]][0]
    tmax = trange[RADARS[0]][1]
    for rd in RADARS:
        if trange[rd][0] < tmin:
            tmin = trange[rd][0]
        if trange[rd][1] > tmax:
            tmax = trange[rd][1]
    # 把tmin往前取整10分钟，把tmax往后取整10分钟
    tmin = tmin - tmin%600
    tmax = tmax + 600 - tmax%600
    # print
    print('All radar has %d files, start time is %s, end time is %s, time range is %d hours'%(len(ft),datetime.fromtimestamp(tmin).strftime('%Y%m%d%H%M'),datetime.fromtimestamp(tmax).strftime('%Y%m%d%H%M'),int((tmax-tmin)/3600)))

    return tmin,tmax


def do_mosaic_qpe(tstep=10):
    # ACC_HOURS = [3],
    # START_LAT= 25,
    # END_LAT  = 30,
    # START_LON= 109,
    # END_LON  = 114,
    # MOSAIC_RESO = 0.01,
    # RADARS = ['Z9734','Z9731','Z9739','Z9746'],
    # PATH_QPE = 'out_qpe', 
    # PATH_QPE_MOSAIC='qpe_mosaic',
    # PATH_TMP='tmp',
    # REDO_MINS=20,

    #如果是历史模式
    if RUNMODE == 1: # 对小时进行累计
        # timestamp类型
        startt,endt = get_time_range_from_filelist()
        # 按拼图间隔，从startt，处理到endt，具体采用do_single函数
        # 每个时刻单独分配一个进程，采用多进程处理
        _pools = Pool(CPU_MAX)
        t = startt
        allt=[]
        while t <= endt:
            #将t转换为datetime类型
            allt.append(datetime.fromtimestamp(t))
            t += MOSAIC_STEP*60
            
        _pools.map(do_single,allt)
        _pools.close()
        _pools.join()
    elif RUNMODE == 2: # 对分钟进行累计
        # timestamp类型
        startt,endt = get_time_range_from_filelist()
        # 按拼图间隔，从startt，处理到endt，具体采用do_single函数
        # 每个时刻单独分配一个进程，采用多进程处理
        _pools = Pool(CPU_MAX)
        t = startt
        allt=[]
        while t <= endt:
            #将t转换为datetime类型
            allt.append(datetime.fromtimestamp(t))
            t += MOSAIC_STEP*60
            
        _pools.map(do_single,allt)
        _pools.close()
        _pools.join()
    else:
        #将curt设置为当前UTC时间
        curt = datetime.utcnow()
        
        do_single(curt)
    
    if not BDEBUG:
        print('Waiting for new data......')
        schedule.enter(tstep, 0, do_mosaic_qpe, (tstep,))

def _delete_old_data(tstep):
    
    print('delete expired data ...')
    curtime = datetime.now()
    pretime = curtime + timedelta(hours=-1*int(data_save_hours))
    
    #查询过期数据
    # curpath = PATH_TMP
    # if not os.path.exists(curpath):
    #     return False
    # ff = os.listdir(curpath)
    # for f in ff:
    #     t = os.path.getctime(curpath + os.sep + f)
    #     if t < pretime.timestamp():
    #         os.remove(curpath + os.sep + f)
    #         print(['Delete file:' + curpath + os.sep + f])

    for acc_hours in ACC_HOURS:          
        curpath = PATH_QPE_MOSAIC + os.sep + '%02dhr'%acc_hours
        if not os.path.exists(curpath):
            return False
        ff = os.listdir(curpath)
        for f in ff:
            t = os.path.getctime(curpath + os.sep + f)
            if t < pretime.timestamp():
                os.remove(curpath + os.sep + f)
                print(['Delete file:' + curpath + os.sep + f])

    schedule.enter(tstep, 0, _delete_old_data, (tstep,))

if __name__ == "__main__":    
    
    print('Usage: python s4_mosaic_qpe.py inifile')
    # 所有参数（包含脚本名）
    all_args = sys.argv  
    # 实际参数（排除脚本名）
    inifile = sys.argv[1:]
    

    #如果未指定配置文件，则选用默认的配置
    if len(inifile) ==0:
        inifile = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/metradar/project/qpe/main_qpe_cfg.ini'
        # inifile = 'archive_main_qpe_cfg.ini'
    else:
        inifile = inifile[0]
    
    config = _get_config_from_rcfile(inifile)
    RUNMODE = int(config['COMMON']['RUN_MODE'])#X方向格点数
    if RUNMODE == 0:
        BDEBUG = False
    else:
        BDEBUG = True

    
    ROOT_PATH = config['PATH_SETTING']['ROOT_PATH']
    BASEDATA_PATH = ROOT_PATH + os.sep + config['PATH_SETTING']['BASEDATA_PATH']
    RADAR_SITES_FILE = config['PATH_SETTING']['RADAR_SITES_FILE']
    RADARS = config['RADAR_SITES']['RADARS'].split(',')
    PATH_QPE = ROOT_PATH + os.sep + config['PATH_SETTING']['PATH_QPE']
    PATH_QPE_MOSAIC = ROOT_PATH + os.sep + config['PATH_SETTING']['PATH_QPE_MOSAIC']
    PATH_TMP = ROOT_PATH + os.sep + config['PATH_SETTING']['PATH_TMP']
    data_save_hours = config['DATA_SAVE_SETTING']['DATA_SAVE_HOURS']
    ACC_HOURS = [int(mm) for mm in config['PARAMS']['acc_hours'].split(',')]
    if not os.path.exists(PATH_QPE_MOSAIC):
        try:
            os.makedirs(PATH_QPE_MOSAIC)
        except:
            print('创建目录失败：' + PATH_QPE_MOSAIC)
            
    CPU_MAX = int(cpu_count() * float(config['PARAMS']['CPU_RATE']))
    #降水产品格点设置
    MOSAIC_RESO = float(config['PARAMS']['MOSAIC_RESO']) #网格分辨率，度
    START_LAT = float(config['PARAMS']['SOUTH_LAT']) #南纬
    END_LAT = float(config['PARAMS']['NORTH_LAT']) #北纬
    START_LON = float(config['PARAMS']['WEST_LON']) #西经
    END_LON = float(config['PARAMS']['EAST_LON']) #东经

    REDO_MINS = int(config['PARAMS']['REDO_MINS'])
    MOSAIC_STEP = int(config['PARAMS']['MOSAIC_STEP'])

    if not os.path.exists(RADAR_SITES_FILE):
        print(RADAR_SITES_FILE + ' 文件不存在，程序退出！')
        exit(-1)
    ALL_RADARS = pd.read_csv(RADAR_SITES_FILE,header=0,index_col=0)
    

    with open('current_pid_s4_%d.txt'%RUNMODE,'wt') as f:
        f.write('current pid is: %s'%str(os.getpid()) + '  ,' + datetime.now().strftime('%Y-%m-%dT%H:%M:%S'))

    
    tstep=10
    
    if not BDEBUG:
        # 增加数据处理任务
        schedule.enter(0, 0, do_mosaic_qpe, (tstep,))    

        # 增加数据管理任务，定时删除旧文件
        schedule.enter(0, 0, _delete_old_data, (60,))

        # 增加程序自动重启任务，以解决可能存在的bug
        schedule.run()
    else:
        do_mosaic_qpe(tstep)

# %%




# _*_ coding: utf-8 _*_

'''
快速绘制三维雷达组网拼图

'''

# %%
import pyart
import os
import numpy as np
import cartopy.crs as ccrs
import matplotlib.pyplot as plt
from cartopy.io.shapereader import Reader
import matplotlib.patheffects as path_effects
from matplotlib.transforms import offset_copy
from matplotlib.font_manager import FontProperties
import pandas as pd
from pyart.graph import common
from datetime import datetime,timedelta
from metradar.util.parse_pal import parse_pro,parse
import xarray as xr

from metradar.config import CONFIG

# 资源文件路径
RESOURCES_PATH = CONFIG.get('SETTING','RESOURCES_PATH')
FONT_FILE = RESOURCES_PATH + '/fonts/YaHeiConsolasHybrid_1.12.ttf'
COLOR_FILE=RESOURCES_PATH + '/gr2_colors/default_BR_PUP2.pal'
MAP_PATH = RESOURCES_PATH + '/国省市县审图号(GS (2019) 3082号/'
STATION_FILE = RESOURCES_PATH + '/stations/cma_city_station_info.dat'


def add_china_map_2cartopy(ax, name='province', facecolor='none',transform=None,
                           edgecolor='c', lw=2, **kwargs):
    """
    Draw china boundary on cartopy map.

    :param ax: matplotlib axes instance.
    :param name: map name.
    :param facecolor: fill color, default is none.
    :param edgecolor: edge color.
    :param lw: line width.
    :return: None
    """

    # map name
    names = {'nation': "BOUL_G", 'province': "BOUL_S",
            'city': "BOUL_D", 'county': "BOUL_X",}

    # add 省界
    if transform is None:
        transform = ccrs.PlateCarree()

    if name == 'county':
        # 添加县边界
        shpfile = MAP_PATH + os.sep + names[name] + ".shp"
        ax.add_geometries(
            Reader(shpfile).geometries(), transform,
            path_effects=[path_effects.Stroke(linewidth=lw, foreground=[1,1,1]),path_effects.Normal()],
            facecolor=facecolor, edgecolor=edgecolor, lw=lw-0.5, **kwargs)

    if name == 'city':
        # 添加市边界
        shpfile = MAP_PATH + os.sep + names[name] + ".shp"
        ax.add_geometries(
            Reader(shpfile).geometries(), transform,
            path_effects=[path_effects.Stroke(linewidth=lw+0.2, foreground=[1,1,1]),path_effects.Normal()],
            facecolor=facecolor, edgecolor=edgecolor, lw=lw-0.2, **kwargs)
        
    # 添加省边界
    if name == 'province':
        shpfile = MAP_PATH + os.sep + names[name] + ".shp"
        ax.add_geometries(
            Reader(shpfile).geometries(), transform,
            path_effects=[path_effects.Stroke(linewidth=lw+0.5, foreground=[1,1,1]),path_effects.Normal()],
            facecolor=facecolor, edgecolor=edgecolor, lw=lw+0.2, **kwargs)
        
def draw_gisinfo(ax,slat,nlat,wlon,elon):

    # 添加中文地名

    city_file = STATION_FILE
  
    cities = pd.read_csv(city_file,  delimiter=r"\s+") # cma_city_station_info
   
    gis_lats=[]
    gis_lons=[]
    gis_name=[]
    for ng in range(len(cities['lon'])):
        gis_lons.append(cities['lon'][ng])
        gis_lats.append(cities['lat'][ng])
        gis_name.append(cities['city_name'][ng])
    

   
    font2=FontProperties(fname=FONT_FILE, size=8)
    geodetic_transform = ccrs.Geodetic()._as_mpl_transform(ax)
    text_transform = offset_copy(geodetic_transform, units='dots', x=-5)
    aa = (np.array(gis_lats)<nlat) & (np.array(gis_lats) > slat)
    bb = (np.array(gis_lons)<elon) & (np.array(gis_lons) > wlon)
    cc = aa & bb
    # cc = cc.values
    for nn in range(len(gis_name)):
        if not cc[nn]:
            continue
        curlat = gis_lats[nn]
        curlon = gis_lons[nn]

        ax.text(curlon, curlat, gis_name[nn], clip_on=True,
                        verticalalignment='center', horizontalalignment='right',
                        transform=text_transform, fontproperties=font2, color='white',
                        path_effects=[path_effects.Stroke(linewidth=1, foreground='black'),path_effects.Normal()])

# 快速绘制垂直剖面图
def quick_draw_mosaic_cross(grid:pyart.core.Grid,params):
    
    # Setting projection, figure size, and panel sizes.
    projection = ccrs.PlateCarree()

    fig = plt.figure(figsize=[16, 7])

    map_panel_axes = [0.1, 0.1, .4, .80]
    x_cut_panel_axes = [0.55, 0.15, .4, .25]
    y_cut_panel_axes = [0.55, 0.55, .4, .25]

    
    # lat = 32.89
    # lon = 118.30
    lat = params['cross_lat']
    lon = params['cross_lon']
    xlim_west = params['xlim_west']
    xlim_east = params['xlim_east']
    ylim_south = params['ylim_south']
    ylim_north = params['ylim_north']

    xcross_left = params['xcross_left']
    xcross_right = params['xcross_right']
    ycross_down = params['ycross_down']
    ycross_up = params['ycross_up']
    zcross_bottom = params['zcross_bottom']
    zcross_top = params['zcross_top']
    dis_level = params['level']

    min_ref = params['min_ref']

    am = grid.fields['reflectivity']['data'].mask
    da = grid.fields['reflectivity']['data'].data
    am[da < min_ref] = True

    display = pyart.graph.GridMapDisplay(grid)
    # Set parameters.
    level = 3
    vmin = 0
    vmax = 75

    cmapname = 'NWSRef'

    # Panel 1: PPI plot of the second tilt.
    ax1 = fig.add_axes(map_panel_axes, projection=projection)
    display.plot_grid('reflectivity', dis_level, vmin=vmin, vmax=vmax,
                    ax=ax1,
                    projection=projection,
                    cmap=cmapname)
    display.plot_crosshairs(lon=lon, lat=lat)
    ax1.set_ylim([ylim_south, ylim_north])
    ax1.set_xlim([xlim_west, xlim_east])
    add_china_map_2cartopy(ax1, name='province', facecolor='none',edgecolor=None, lw=1)
    draw_gisinfo(ax1,slat=ylim_south,nlat=ylim_north,wlon=xlim_west,elon=xlim_east)
    # Panel 2: longitude slice
    ax2 = fig.add_axes(x_cut_panel_axes)
    display.plot_longitude_slice('reflectivity', lon=lon, lat=lat,
                                ax=ax2,
                                vmin=vmin, vmax=vmax,
                                cmap=cmapname)

    ax2.set_ylim([zcross_bottom, zcross_top])
    ax2.set_xlim([xcross_left, xcross_right])

    # Panel 3: latitude slice
    ax3 = fig.add_axes(y_cut_panel_axes)
    display.plot_latitude_slice('reflectivity', lon=lon, lat=lat,
                                ax=ax3,
                                vmin=vmin, vmax=vmax,
                                cmap=cmapname)
    ax3.set_ylim([zcross_bottom, zcross_top])
    ax3.set_xlim([ycross_down, ycross_up])
    plt.close()


def draw_composite_operational(gridfile,outpath='.',level=4,vmin=0,vmax=75,colorfile=None):

    if not os.path.exists(gridfile):
        print(gridfile + ' not exists!')
        return False
    try:
        grid = pyart.io.read_grid(gridfile)
    except:
        print(gridfile + ' read error!')
        return False
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    (filepath,filename) = os.path.split(gridfile)
    tstr = filename[7:19]
    tt = datetime.strptime(tstr,'%Y%m%d%H%M') + timedelta(hours=8)

    tstr_date = '%04d年%02d月%02d日'%(tt.year,tt.month,tt.day)
    tstr_time = '%02d时%02d分（北京时）'%(tt.hour,tt.minute)
    font_large=FontProperties(fname=FONT_FILE, size=14)
    font_mid=FontProperties(fname=FONT_FILE, size=10)

    min_ref=10
    am = grid.fields['reflectivity']['data'].mask
    da = grid.fields['reflectivity']['data'].data
    am[da < min_ref] = True
    xrange = grid.x['data'].max()
    yrange = grid.y['data'].max()
    origin_lat = grid.origin_latitude['data'][0]
    origin_lon = grid.origin_longitude['data'][0]
    [startlon,startlat]=pyart.core.cartesian_to_geographic_aeqd(-1*xrange,-1*yrange,origin_lon,origin_lat)
    [endlon,endlat]=pyart.core.cartesian_to_geographic_aeqd(1*xrange,1*yrange,origin_lon,origin_lat)

    display = pyart.graph.GridMapDisplay(grid)


    ax_pos_main   = [0.05, 0.05, .65, .65]
    ax_pos_top    = [0.05, 0.71, .65, .2] 
    ax_pos_right  = [0.71, 0.05, .2, .65]
    ax_pos_cbar   = [0.92, 0.05, .03, .65]


    fig = plt.figure(figsize=(8,8))
    projection = ccrs.PlateCarree()
    ax_main = fig.add_axes(ax_pos_main, projection=projection)
    
    fig.text(0.5,0.97, '三维雷达组网拼图-多视角组合图', va="center", ha="center", font=font_large ) 
    
    # add product info
    infostr = '要 素：' + '反射率因子'
    fig.text(0.71,0.89, infostr, va="center", ha="left", font=font_mid ) 
    infostr = '日 期：' + tstr_date
    fig.text(0.71,0.86, infostr, va="center", ha="left", font=font_mid ) 
    infostr = '时 间：' + tstr_time
    fig.text(0.71,0.83, infostr, va="center", ha="left", font=font_mid ) 
    infostr = '高 度：%d米(左下图)'%grid.z['data'][level]
    fig.text(0.71,0.80, infostr, va="center", ha="left", font=font_mid ) 

    ax_main.axis('tight')
    
    xlim_west = startlon[0]
    xlim_east = endlon[0]
    ylim_south = startlat[0]
    ylim_north = endlat[0]
    
    
    field = 'reflectivity'
    if colorfile is None:
        cmapname = 'NWSRef'
        cmap = common.parse_cmap(cmapname, field)
    else:
        cmap,norm=parse(colorfile)
        # outdic= parse(colorfile)
        # cmap=outdic['cmap']
        # norm=outdic['norm']

    display.plot_grid(field, level, vmin=vmin, vmax=vmax,
                    ax=ax_main,colorbar_flag=False,title_flag=False,
                    projection=projection,axislabels_flag=False,title=None,
                    cmap=cmap,norm=norm)
    add_china_map_2cartopy(ax_main, name='province', facecolor='none',edgecolor=None, lw=1)
    draw_gisinfo(ax_main,slat=ylim_south,nlat=ylim_north,wlon=xlim_west,elon=xlim_east)
    
    infostr = '基于 metradar 制作'

    fig.text(0.05,0.03, infostr, va="center", ha="left", font=font_mid ) 

    ax_main.set_ylim([ylim_south, ylim_north])
    ax_main.set_xlim([xlim_west, xlim_east])
    ax_main.set_xticks([])
    ax_main.set_yticks([])
    ax_main.set_xlabel('')
    ax_main.set_ylabel('')
    plt.title('')

    ax_top = fig.add_axes(ax_pos_top)
    # ax_top.axis('equal')
    x_1d = grid.x['data'] / 1000
    y_1d = grid.y['data'] / 1000
    z_1d = grid.z['data'] / 1000
    # data = self.grid.fields[field]['data'][:, y_index, :]
    # data = np.ma.masked_outside(data, vmin, vmax)

    ref = grid.fields[field]['data']
    data1 = np.max(ref,1)
    xd, zd = np.meshgrid(x_1d, z_1d)
    pm_top = ax_top.pcolormesh(
            xd, zd, data1, vmin=vmin, vmax=vmax,cmap=cmap )
    ax_top.set_ylim([0, 20])
    plt.yticks([0,5,10,15,20])
    plt.xticks([])
    plt.title('经度-高度图（公里）（沿经向取最大值）',fontproperties=font_mid)
    plt.grid(color='k',axis='y')
    
    ax_right = fig.add_axes(ax_pos_right)
    data2 = np.max(ref,2)
    zd, xd = np.meshgrid(z_1d, x_1d)
    pm_right = ax_right.pcolormesh(
            zd, xd, data2.T, vmin=vmin, vmax=vmax,cmap=cmap )
    ax_right.set_xlim([0, 20])
    plt.xticks([0,5,10,15,20])
    plt.yticks([])
    plt.title('纬度-高度图（公里）\n（沿纬向取最大值）',fontproperties=font_mid)
    plt.grid(color='k',axis='x')

    # add colorbar
    ax_cbar = fig.add_axes(ax_pos_cbar)
    cb=plt.colorbar(pm_right,cax=ax_cbar,orientation='vertical')#方向
    plt.title('dBZ')
    plt.savefig(outpath + os.sep + filename.replace('.nc','.png'),dpi=600)
    plt.close()
    # plt.show()
    # create axis
    print('save figure to ' + outpath + os.sep + filename.replace('.nc','.png'))
    pass

if __name__ == "__main__":

    
    path = '/mnt/e/metradar_test/vpr/mosaic/20230731_daxing'
    outpath = '/mnt/e/metradar_test/vpr/mosaic/20230731_daxing/mosac_pic'

    print(os.listdir(path))
    for file in os.listdir(path):
        if not file.endswith('.nc'):
            continue
        draw_composite_operational(path + os.sep + file,outpath,colorfile=None)
        pass





'''
读取新的雷达拼图数据
朱文剑


数据格式说明

//
/////////////////////////////////////////////////////////////////////
//  组网定量估测降水系统拼图产品文件规则
//  1. 文件里命名为:
//		ACHN_XXX_20121012_090000_nn.bin
// 2. 文件格式:文件头 + 数据块 
// 3. 数据从西北，往东南写
//***************************************
typedef struct tagMQPE_MosiacProductHeaderStructure
{
	// 文件卷标，12字节(0-11), 
	char  label[4];	    //0-3	文件固定标识：MOC 
	char  Version[4];	//4-7	文件格式版本代码，如: 1.0，1.1，etc
	int   FileBytes;	//8-11	包含头信息在内的文件字节数，不超过2M

						//产品描述, 84字节(12-95) 
	short MosaicID;		  //12-13 拼图产品编号
	short coordinate;	  //14-15 坐标类型: 2=笛卡儿坐标,3=等经纬网格坐标
	char  varname[8];     //16-23 产品代码,如: ET,VIL,CR,CAP,OHP,OHPC
	char  description[64];//24-87 产品描述,如Composite Reflectivity mosaic
	int   BlockPos; //88-91 产品数据起始位置(字节顺序)
	int   BlockLen; //92-95  产品数据字节数

					//数据时间, 28字节(96-123) 
	int TimeZone;   //96-99 数据时钟,0=世界时,28800=北京时
	short yr;		//100-101	观测时间中的年份
	short mon;	    //102-103	观测时间中的月份（1－12）
	short day;		//104-105	观测时间中的日期（1－31）
	short hr;		//106-107	观测时间中的小时（00－23）
	short min;	    //108-109	观测时间中的分（00－59）
	short sec;	    //110-111	观测时间中的秒（00－59）
	int   ObsSeconds; //112-115	观测时间的seconds 
	unsigned short ObsDates; //116-117 观测时间中的Julian dates
	unsigned short GenDates; //118-119 产品处理时间的天数
	int   GenSeconds;   //120-123 产品处理时间的描述

						//数据区信息, 48字节(124-171) 
	int  edge_s;  //124-127 数据区的南边界，单位：1/1000度，放大1千倍
	int  edge_w;  //128-131 数据区的西边界，单位：1/1000度，放大1千倍
	int  edge_n;  //132-135 数据区的北边界，单位：1/1000度，放大1千倍
	int  edge_e;  //136-139 数据区的东边界，单位：1/1000度，放大1千倍
	int  cx;      //140-143 数据区中心坐标，单位：1/1000度，放大1千倍
	int  cy;      //144-147 数据区中心坐标，单位：1/1000度，放大1千倍
	int  nX;      //148-151 格点坐标为列数,
	int  nY;      //152-155 格点坐标为行数,
	int  dx;      //156-159 格点坐标为列分辨率，单位：1/10000度，放大1万倍
	int  dy;      //160-163 格点坐标为行分辨率，单位：1/10000度，放大1万倍
	short height; //164-165 雷达高度
	short Compress; //166-167 数据压缩标识, 0=无,1=bz2,2=zip,3=lzw
	int   num_of_radars; //168-171有多少个雷达进行了拼图

	int UnZipBytes; //数据段压缩前的字节数, //172-175
	short scale;     //176-177
	short unUsed; //178-179
	char  RgnID[8];  //180-187
	char units[8];  //180-187
	char  reserved[60];
}MOSAIC_PRODUCTHEADER;

-32768表示空白


'''

# %%
CODE1 = 'B'
CODE2 = 'H'
INT1 = 'B'
INT2 = 'H'
INT4 = 'I'
REAL4 = 'f'
REAL8 = 'd'
SINT1 = 'b'
SINT2 = 'h'
SINT4 = 'i'

MOSAIC_HEADER = (
    ('label', '4s'),
    ('Version', '4s'),
    ('FileBytes', INT4),
    ('MosaicID', INT2),
    ('coordinate', INT2),
    ('varname', '8s'),
	('description', '64s'),
    ('BlockPos', INT4),
    ('BlockLen', INT4),
    ('TimeZone', INT4),
    ('year', INT2),
    ('month', INT2),
	('day', INT2),
    ('hour', INT2),
    ('min', INT2),
    ('sec', INT2),
    ('ObsSeconds', INT4),
    ('ObsDates', INT2),
	('GenDates', INT2),
    ('GenSeconds', INT4),
    ('edge_s', INT4),
    ('edge_w', INT4),
    ('edge_n', INT4),
    ('edge_e', INT4),
	('cx', INT4),
    ('cy', INT4),
    ('nX', INT4),
    ('nY', INT4),
    ('dx', INT4),
    ('dy', INT4),
	('height', INT2),
    ('Compress', INT2),
    ('num_of_radars', INT4),
    ('UnZipBytes', INT4),
    ('scale', INT2),
    ('unUsed', INT2),
	('RgnID', '8s'),
	('units', '8s'),
	('reserved', '60s'),

)

import os
import numpy as np
import struct
import bz2
import xarray as xr
from datetime import datetime,timedelta


from multiprocessing import cpu_count, Pool,freeze_support

def _structure_size(structure):
        """ Find the size of a structure in bytes. """
        return struct.calcsize('<' + ''.join([i[1] for i in structure]))


def _unpack_from_buf( buf, pos, structure):
    """ Unpack a structure from a buffer. """
    size = _structure_size(structure)
    return _unpack_structure(buf[pos:pos + size], structure)


def _unpack_structure(string, structure):
    """ Unpack a structure from a string """
    fmt = '<' + ''.join([i[1] for i in structure])  # little-endian
    lst = struct.unpack(fmt, string)
    return dict(zip([i[0] for i in structure], lst))

def decode_mosaic(filepath,filename,minv=10,maxv = 80):

	if not os.path.exists(filepath+os.sep + filename):
		print(filepath+os.sep+filename + ' not exists!')
		return None,None
	try:
		fin = open(filepath + os.sep + filename,'rb')
	except:
		return None,None

	buf = fin.read()
	fin.close()
	pos = 0
	if len(buf)==0:
		return None
	dic_gh = _unpack_from_buf(buf, pos, MOSAIC_HEADER)
	pos +=_structure_size(MOSAIC_HEADER)
	databuf = bz2.decompress(buf[pos:])
	data = np.frombuffer(databuf, 'int16')
	data = np.reshape(data,[dic_gh['nY'],dic_gh['nX']])
	data=data.astype('float')
	# data[data == -32768] = np.NAN #  空白
	# data[data == -1280] = np.NAN # 有覆盖但无数据
	data[data < -300] = np.NAN
	data = np.flipud(data)/dic_gh['scale']
	data[data < int(minv)] = np.NAN
	data[data > int(maxv)] = np.NAN

	# set longitude and latitude coordinates
	lat = dic_gh['edge_s']/1000 + np.arange(dic_gh['nY'])*dic_gh['dy']/10000
	lon = dic_gh['edge_w']/1000 + np.arange(dic_gh['nX'])*dic_gh['dx']/10000

	# set time coordinates
	time = datetime(dic_gh['year'],dic_gh['month'],dic_gh['day'],dic_gh['hour'],dic_gh['min'],dic_gh['sec'])
	time = np.array([time], dtype='datetime64[m]')
	# data = np.expand_dims(data, axis=0)
	tmpdata = np.reshape(data, (1, data.shape[0], data.shape[1]))
	# define coordinates
	lev_coord = ('lev', [dic_gh['height'],], {'long_name':'height', 'units':'m', '_CoordinateAxisType':'Height'})
	lon_coord = ('lon', lon, {
	    'long_name':'longitude', 'units':'degrees_east', '_CoordinateAxisType':'Lon'})
	lat_coord = ('lat', lat, {
	    'long_name':'latitude', 'units':'degrees_north', '_CoordinateAxisType':'Lat'})

	# create xarray
	varattrs = {'long_name': 'Refelectivity', 
	            'short_name': 'ref', 'units': 'dBZ',
				'maxv':maxv,
				'minv':minv}
	data = xr.Dataset({'ref':(['lev','lat', 'lon'], tmpdata, varattrs)},coords={ 'lev':lev_coord,'lat':lat_coord, 'lon':lon_coord})

	# add attributes
	data.attrs['Conventions'] = "CF-1.6"
	data.attrs['Origin'] = 'cmadaas'
	data.attrs['author'] = 'ZWJ'
	
	return data

def trans_single(param):

	filepath = param['filepath']
	filename = param['filename']
	outpath = param['outpath']
	pass
	varflag = 'unknown'
	if filename.find('CREF') > 0:
		varflag = 'CREF000'
	elif filename.find('QREF') > 0:
		varflag = 'QREF000'
	elif filename.find('_CAP_') > 0:
		varflag = 'CAP'
	if varflag == 'CAP':
		outname = 'ACHN.' + varflag + '.' + filename.split('_')[4][0:8] +'.' + \
			       filename.split('_')[4][8:14] + '_' + filename.split('_')[-1].split('.')[0] + '.nc'
	else:
		# outname = 'ACHN.' + varflag + '.' + filename.split('_')[4][0:8] +'.' + filename.split('_')[4][8:14] + '.nc'
		outname = 'ACHN.' + varflag + '.' + filename.split('_')[2] +'.' + filename.split('_')[3][0:6] + '.nc'
	if os.path.exists(outpath + os.sep + outname):
		print(outpath + os.sep + outname + ' already exists!')
		return True
	try:
		data=decode_mosaic(filepath,filename,minv=0,maxv=80)
		if data is None:
			return False
		datakey = list(data.keys())
		if 'cref' not in datakey:
			#将ref变量rename为cref
			data=data.rename({datakey[0]:'cref'})
		if not os.path.exists(outpath):
			os.makedirs(outpath)
		data.to_netcdf(outpath + os.sep + outname,encoding={'cref': {'zlib': True}})
		print(outpath + os.sep + outname + ' done!')
	except Exception as e:
		print(e)
		print(outpath + os.sep + outname + ' make error!')

# 批量转nc文件
def trans_bin_nc_batch(inpath,outpath,):

	if not os.path.exists(inpath):
		print(inpath + ' not exists!')
	if not os.path.exists(outpath):
		os.makedirs(outpath)

	files = os.listdir(inpath)
	files=sorted(files)
	if len(files)==0:
		print('no valid files')
		return False
	
	params=[]
	for filename in files:
		if filename.startswith('.') or filename.startswith('..'):
			continue
		if not filename.endswith('.bin'):
			continue
		curparam=dict()
		curparam['filepath'] = inpath
		curparam['filename'] = filename
		curparam['outpath'] = outpath# + os.sep + filename.replace('.bin','.nc')#filename.split('_')[9][0:4]
		params.append(curparam)
		

	# 多进程处理
	MAXP = int(cpu_count()*0.6)
	pools = Pool(MAXP)

	pools.map(trans_single, params)
	pools.close()
	pools.join()

if __name__ == '__main__':


	sourcepath = r'/Users/wenjianzhu/Downloads/CMADAAS/rdmosaic_bin'
	outpath = r'/Users/wenjianzhu/Downloads/CMADAAS/mosaicdata_nc'
	starttime = '20230701'
	endtime = '20230702'
	startt = datetime.strptime(starttime,'%Y%m%d')
	endt = datetime.strptime(endtime,'%Y%m%d')
	curt = startt
	# ACHN.CREF000.20221001.143000.nc
	prepath = ''
	while curt <= endt:
		pass
		curpath = sourcepath + os.sep + curt.strftime('%Y%m')
		if prepath == curpath:
			curt  = curt + timedelta(days=1)
			continue
		if not os.path.exists(curpath):
			curt  = curt + timedelta(days=1)
			continue
		
		trans_bin_nc_batch(curpath,outpath)
		curt  = curt + timedelta(days=1)
		prepath = curpath



# _*_ coding: utf-8 _*_
'''
绘制雷达拼图
'''


# %%
import os
import matplotlib.pyplot as plt 
import cartopy.crs as ccrs
from metradar.util.parse_pal import parse,parse_pro
import xarray as xr
import numpy as np
from matplotlib.transforms import offset_copy
import matplotlib.patheffects as path_effects
from matplotlib.font_manager import FontProperties
import json
import matplotlib as mpl
import matplotlib
matplotlib.use('agg')

from cartopy.io.shapereader import Reader
import pandas as pd
from multiprocessing import cpu_count, Pool,freeze_support
from metradar.config import CONFIG

# 资源文件路径
RESOURCES_PATH = CONFIG.get('SETTING','RESOURCES_PATH')

FONT_FILE = RESOURCES_PATH + '/fonts/YaHeiConsolasHybrid_1.12.ttf'
COLOR_PATH=RESOURCES_PATH + '/gr2_colors/'
MAP_PATH = RESOURCES_PATH + '/国省市县审图号(GS (2019) 3082号'
STATION_FILE = RESOURCES_PATH + '/stations/cma_city_station_info.dat'

g_font_max = FontProperties(fname= FONT_FILE, size=8)
g_font_mid = FontProperties(fname= FONT_FILE, size=6)
g_font_min = FontProperties(fname= FONT_FILE, size=4)

mpl.rcParams['font.size'] = 7
mpl.rcParams['font.weight'] = 'normal'
# mpl.rcParams['font.family'] = 'Times New Roman'


def draw_gisinfo(ax,font=None):

    city_file =STATION_FILE
    
    if not os.path.exists(city_file):
        print(city_file +' not exist!')
        return False

    cities = pd.read_csv(city_file,  delimiter=r"\s+") # cma_city_station_info

    cityname = np.unique(cities['city_name'])
    names=[]
    lat=[]
    lon=[]

    curdata = cities
    for name in cityname:
        flag2 =  curdata['city_name'] == name
        tmp_lat = np.mean(curdata[flag2]['lat'])
        tmp_lon = np.mean(curdata[flag2]['lon'])

        names.append(name)
        lon.append(tmp_lon)
        lat.append(tmp_lat)

    cities = pd.DataFrame(data={'name':names, 'lat':lat, 'lon':lon})
    # draw station information
    font = g_font_min
    geodetic_transform = ccrs.Geodetic()._as_mpl_transform(ax)
    for _, row in cities.iterrows():
        text_transform = offset_copy(geodetic_transform, units='dots', x=-5)
        ax.plot(row['lon'], row['lat'], marker='o', color=[0.2,0.2,0.2], linestyle='None',
                markersize=2, alpha=0.8, transform=ccrs.PlateCarree())
        ax.text(row['lon']-0.05, row['lat'], row['name'], clip_on=True, # city_name
                verticalalignment='center', horizontalalignment='right',
                transform=text_transform, fontproperties=font, color=[0.2,0.2,0.2],
                path_effects=[path_effects.Stroke(linewidth=1, foreground=[1,1,1]),
                path_effects.Normal()])
                
    
def add_china_map_2cartopy(ax, facecolor='none',transform=None,name='province',
                           edgecolor='k', lw=0.8, **kwargs):
    """
    Draw china boundary on cartopy map.

    :param ax: matplotlib axes instance.
    :param name: map name.
    :param facecolor: fill color, default is none.
    :param edgecolor: edge color.
    :param lw: line width.
    :return: None
    """

    # map name
    names = {'nation': "BOUL_G", 'province': "BOUL_S",
            'city': "BOUL_D", 'county': "BOUL_X",}

    if not name in {'nation','province','city','county'}:
        print(name + ' should be nation or province or city or county !')
        return False


    # add 省界
    if transform is None:
        transform = ccrs.PlateCarree()

    if name == 'county':
        # 添加县边界
        shpfile = MAP_PATH + os.sep + names[name] + ".shp"
        ax.add_geometries(
            Reader(shpfile).geometries(), transform,
            path_effects=[path_effects.Stroke(linewidth=lw, foreground=[1,1,1]),path_effects.Normal()],
            facecolor=facecolor, edgecolor=edgecolor, lw=lw-0.5, **kwargs)

    if name == 'city':
        # 添加市边界
        shpfile = MAP_PATH + os.sep + names[name] + ".shp"
        ax.add_geometries(
            Reader(shpfile).geometries(), transform,
            path_effects=[path_effects.Stroke(linewidth=lw+0.2, foreground=[1,1,1]),path_effects.Normal()],
            facecolor=facecolor, edgecolor=edgecolor, lw=lw-0.2, **kwargs)
        
    # 添加省边界
    if name == 'province':
        shpfile = MAP_PATH + os.sep + names[name] + ".shp"
        ax.add_geometries(
            Reader(shpfile).geometries(), transform,
            path_effects=[path_effects.Stroke(linewidth=lw+0.5, foreground=[1,1,1]),path_effects.Normal()],
            facecolor=facecolor, edgecolor=edgecolor, lw=lw+0.2, **kwargs)

def cm_precip():
    """
    Standardized colormaps from National Weather Service
    https://github.com/blaylockbk/pyBKB_v2/blob/master/BB_cmap/NWS_standard_cmap.py

    Range of values:
        metric: 0 to 762 millimeters
        english: 0 to 30 inches
    """
    # The amount of precipitation in inches
    # a = [0,.01,.1,.25,.5,1,1.5,2,3,4,6,8,10,15,20,30]
    a = [0,0.01,0.5,2.5,5,10,15,20,30,40,60,80,100,150,200,250]

    # Normalize the bin between 0 and 1 (uneven bins are important here)
    norm = [(float(i)-min(a))/(max(a)-min(a)) for i in a]

    # Color tuple for every bin
    C = np.array(# [255,255,255],
        [ [199,233,192], [161,217,155], [116,196,118], [49,163,83], 
        [0,109,44], [255,250,138], [255,204,79], [254,141,60], [252,78,42], 
        [214,26,28], [173,0,38], [112,0,38], [59,0,48], [76,0,115], [255,219,255]])

    cmap, norm = mpl.colors.from_levels_and_colors(np.array(a), np.array(C)/255.0, extend='neither')
    return cmap, norm


def draw_latlon(data,lat,lon,slat=None,nlat=None,wlon=None,elon=None,outpath=None,outname=None,tstr=None,
                prefix_title='雷达组合反射率拼图 ',units='dBZ',add_title=False,badd_gis=True,badd_logo=True,
                subtitle='',titlecolor='r',thred=None,cb_ratio=0.618,fig_size=[6,4],dpi=400,colorfile=None):
    '''
    该函数在绘制经纬度网格数据的核心函数
    draw_latlon 的 Docstring
    
    :param data: DataArray 类型，二维数组
    :param lat: 数据的纬度列表，一维数组
    :param lon: 数据的经度列表，一维数组
    :param slat: 绘图南边界
    :param nlat: 绘图北边界
    :param wlon: 绘图西边界
    :param elon: 绘图东边界
    :param outpath: 图片保存路径
    :param outname: 图片保存名称
    :param tstr: 时间字符串，YYYYMMDDHHmmss格式
    :param prefix_title: 标题前缀
    :param units: 单位
    :param add_title: 是否添加标题
    :param badd_gis: 是否添加地理信息
    :param badd_logo: 是否在左下角添加说明: Powered by metradar
    :param subtitle: 标题后缀
    :param titlecolor: 标题颜色
    :param thred: 显示阈值，小于该阈值不显示
    :param cb_ratio: 色标长度占Y轴长度的比例
    :param fig_size: 图片画布尺寸，默认为[4,3]
    :param dpi: 保存图片的分辨率
    :param colorfile: 自定义配色文件路径，如果不提供则使用默认配色方案,色标文件必须是.pal格式，可以使用resources目录下的配色文件
    '''
    if units == 'dBZ':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            var_cmap,norm = parse(COLOR_PATH + '/BR_WDTB_Bright.pal')#BR_WDTB_Bright.pal  default_BR_PUP2.pal BR_AVL_BroadcastNegatives.pal
    elif units == 'mm':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            var_cmap,norm = cm_precip()
    elif units == '%':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            var_cmap,norm = parse(COLOR_PATH + '/default_CC_ROSE.pal')
    elif units == 'dB':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            var_cmap,norm = parse(COLOR_PATH + '/default_ZDR_ROSE.pal')
    elif units == 'deg/km':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            var_cmap,norm = parse(COLOR_PATH + '/default_KDP_ROSE.pal')
    elif units == 'cat':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            var_cmap,norm = parse(COLOR_PATH + '/default_HCAS.pal')
    elif units == 'm/s':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            var_cmap,norm = parse(COLOR_PATH + '/default_BV_PUP2.pal')
    elif units == 'kg/m2':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            var_cmap,norm = parse(COLOR_PATH + '/default_VIL.pal')
    elif units == 'km':
        if colorfile is not None:
            var_cmap,norm = parse(colorfile)
        else:
            outdic = parse_pro(COLOR_PATH + '/default_ET_PUP.pal')
            var_cmap,norm = outdic['cmap'],outdic['norm']
    else:
        pass
        
    # newdata.data.shape[1],newdata.data.shape[0]
    fig= plt.figure(figsize=fig_size,dpi=dpi,clear=True)
    map_panel_axes = [0.1, 0.1, .8, .8]

    projs = ccrs.PlateCarree()
    
    ax = plt.axes(map_panel_axes, projection=projs)
    # 如果slat,nlat,wlon,elon没有提供，则使用数据的范围
    if slat is None:
        slat = float(lat.min())
    if nlat is None:
        nlat = float(lat.max())
    if wlon is None:
        wlon = float(lon.min())
    if elon is None:
        elon = float(lon.max())
    extent = [wlon, elon, slat, nlat]
    ax.set_extent(extent, crs=projs) 


    if not thred is None:
        newdata = np.ma.masked_array(data, data < thred)
    else:
        newdata = data
    grid_lat,grid_lon=np.meshgrid(lat,lon)
    pcm = ax.pcolormesh(grid_lon,grid_lat,newdata.T,cmap=var_cmap,norm=norm,transform=projs)


    # add_china_map_2cartopy(ax,name='county')
    # add_china_map_2cartopy(ax,name='city')
    add_china_map_2cartopy(ax,name='province',lw=0.5)

    if badd_gis:
        draw_gisinfo(ax)


    # 添加地图网格
    # ax.minorticks_on()
    # ax.tick_params(direction='out', length=6, width=2, colors='r',grid_color='r', grid_alpha=0.5)
    if elon - wlon > 20:
        lon_step = 5
    elif elon - wlon > 5:
        lon_step = 1
    elif elon - wlon > 1:
        lon_step = 0.5
    else:
        lon_step = 0.2

    if nlat - slat > 20:
        lat_step = 5
    elif nlat - slat > 5:
        lat_step = 1
    elif nlat - slat > 1:
        lat_step = 0.5
    else:
        lat_step = 0.2

    lb=ax.gridlines(draw_labels=True,x_inline=False, y_inline=False,xlocs=np.arange(int(wlon)-1,int(elon)+1,lon_step), ylocs=np.arange(int(slat)-1,int(nlat)+1,lat_step),
                    linewidth=0.1, color='k', alpha=0.8, linestyle='--')
    
    # 在左下角添加一行字： Powered by metradar
    if badd_logo:
        textstr = 'Powered by metradar'
        ax.text(0.01, 0.01, textstr, transform=ax.transAxes, fontsize=5, verticalalignment='bottom',
                horizontalalignment='left', color='k', alpha=0.6, fontproperties=g_font_min)
    # point_lat = 30.72
    # point_lon = 108.57
    # ax.plot(point_lon,point_lat,marker='o',color='r',markersize=4,transform=ccrs.PlateCarree())
    lb.top_labels = None
    lb.right_labels = None
    lb.rotate_labels = False
 
    titlestr1 = prefix_title + tstr[0:8] + 'T' + tstr[8:14] + ' UTC ' + subtitle

    if add_title:
        ax.set_title(titlestr1,fontproperties=g_font_mid,loc='left',verticalalignment='top',color=titlecolor)
    # add colorbar
    ax_pos = ax.get_position().bounds
    cbar_axes = [ax_pos[0]+ ax_pos[2]+0.005,(ax_pos[1]+ax_pos[1]+ax_pos[3])/2-ax_pos[3]*cb_ratio/2,0.02,ax_pos[3]*cb_ratio]
    cb_ax = fig.add_axes(cbar_axes)
    if units == 'cat':
        plt.colorbar(mappable=pcm,cax=cb_ax,extend='both',ticks=np.arange(0.5,11.5,1)) # extend='both',ticks=np.arange(-5,80,5)
    elif units == 'dBZ':
        plt.colorbar(mappable=pcm,cax=cb_ax,extend='both',ticks=np.arange(0,80,5))
    else:
        plt.colorbar(mappable=pcm,cax=cb_ax,extend='both')
    cb_ax.text(1.8, 1.035, units, transform=cb_ax.transAxes,va="center", ha="center", font=g_font_mid ) # 0.5,1.02

    if units == 'cat':
        cb_ax.set_yticklabels(['空','地物','晴空','干雪','湿雪','冰晶','霰','大雨滴','小到中雨','大雨','冰雹',],font=g_font_min)

    
    if not os.path.exists(outpath):
        os.makedirs(outpath,exist_ok=True)
    if os.path.exists(outpath):
        if outname.endswith('.eps'):
            plt.savefig(outpath + os.sep  + outname, transparent=False, dpi=dpi,papertype='a4')
        else:
            fig.savefig(outpath + os.sep  + outname, bbox_inches="tight",transparent=False, dpi=dpi,)
        
        print(outpath + os.sep + outname + ' saved!')
    

def draw_single(param):
    filename = param['filename']
    filepath = param['filepath']
    outpath = param['outpath']
    outname = param['outname']
    tstr = param['timestr']
    slat = param['slat']
    nlat = param['nlat']
    wlon = param['wlon']
    elon = param['elon']
    dpi = param['dpi']
    thred = param['thred']
    

    # outname = filename.replace('.nc','.png')
    # tstr = filename.split('.')[2] + filename.split('.')[3]
    data = xr.open_dataset(filepath + os.sep + filename)
    try:
        newdata = data.cref.sel(latitude=slice(slat,nlat),longitude=slice(wlon,elon))
        draw_latlon(newdata,newdata.latitude.data,newdata.longitude.data,slat,nlat,wlon,elon,outpath,outname,tstr,dpi=dpi,thred=thred)
    except:
        newdata = data.cref.sel(lat=slice(slat,nlat),lon=slice(wlon,elon))
        draw_latlon(newdata,newdata.lat.data,newdata.lon.data,slat,nlat,wlon,elon,outpath,outname,tstr,dpi=dpi,thred=thred)
    
    
    

if __name__ == "__main__":      
    freeze_support()
    filepath = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/testdata/mosaic_bin'
    filename = 'ACHN_CREF_20210918_155000.BIN'
    outpath = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/output/pic'
    
    if not os.path.exists(outpath):
        os.makedirs(outpath)
     # 单张图绘制
    curparam=dict()
    curparam['filepath'] = filepath
    curparam['filename'] = filename
    curparam['outpath'] = outpath
    curparam['outname'] = filename + '.png'
    curparam['timestr'] = filename.split('_')[2] + filename.split('_')[3].split('.')[0]
    curparam['slat'] = 15
    curparam['nlat'] = 55
    curparam['wlon'] = 72
    curparam['elon'] =140
    curparam['dpi'] = 300
    curparam['thred'] = 10

    draw_single(curparam)

    
    
    # 多进程批量绘制
    # params=[]
    # files = os.listdir(filepath)
    # files = sorted(files)
    # for filename in files:
    #     if filename.startswith('.') or filename.startswith('..'):
    #         continue
    #     # if filename.split('.')[3][2:4] != '00' and filename.split('.')[3][2:4] != '12' and filename.split('.')[3][2:4] != '30' and filename.split('.')[3][2:4] != '48':
    #     #     continue
    #     curparam=dict()
    #     curparam['filepath'] = filepath
    #     curparam['filename'] = filename
    #     curparam['outname'] = filename + '.png'
    #     curparam['timestr'] = filename.split('_')[2] + filename.split('_')[3].split('.')[0]
    #     curparam['outpath'] = outpath
    #     curparam['slat'] = 28
    #     curparam['nlat'] = 32.5
    #     curparam['wlon'] = 105.5
    #     curparam['elon'] =110.5
    #     curparam['dpi'] = 300
    #     curparam['thred'] = 15

    #     params.append(curparam)

    # MAXP = int(cpu_count()*0.8)
    # pools = Pool(MAXP)

    # pools.map(draw_single, params)
    # pools.close()
    # pools.join()






from PIL import Image
import os


filepath = 'pic'
outpath = 'pic'
filenames = os.listdir(filepath)
filenames = sorted(filenames)
images = []



for filename in filenames:
    if filename.startswith('.') or filename.startswith('..'):
        continue
    if not filename.endswith('.png'):
        continue

    images.append(Image.open(filepath + os.sep + filename))
    print(filename + ' added!')
images[0].save(outpath + os.sep + '雷达外推示例.gif', format='GIF',
                    append_images=images[1:], save_all=True, duration=int(500), loop=0)




# _*_ coding: utf-8 _*_


'''
该脚本是用于批量绘制综合体，并可以针对不同case进行不同的绘图设置，具体参数保存在类似radardrawlist_20120612.csv文件中

朱文剑
'''

import pandas as pd
import warnings
warnings.filterwarnings('ignore')

from metradar.graph.draw_radar_comp_func import DRAW_RADAR_OTHER,ini_params
from multiprocessing import cpu_count, Pool,freeze_support


def draw_all(params):
    # sourcepath,outpath,filename,start_lat,end_lat,start_lon,end_lon
    # 河南北部区域
    sourcepath = params['sourcepath']
    outpath = params['outpath']
    radfilename = params['radfilename']
    slat = params['slat']
    nlat = params['nlat']
    wlon = params['wlon']
    elon = params['elon']

    timestr = radfilename[5:13] + '.' +  radfilename[14:20]

    
    # 用户需根据实际情况设置路径
    fontname='../common/fonts/msyhbd.ttc'

    # 添加中文地名
    # 用户需根据实际情况设置路径
    filename = '../common/中文地理信息原始文件/吉林省所有站点信息.xls'
  
    data_guojia = pd.read_excel(filename, sheet_name = '国家站55',header=0,skiprows=0,index_col=None,)
    for jj in range(data_guojia.shape[0]):
        curidx = data_guojia['站名'][jj].find('国家')
        data_guojia['站名'][jj] = data_guojia['站名'][jj].replace( data_guojia['站名'][jj][curidx:],'')
        data_guojia['站名'][jj] = data_guojia['站名'][jj].replace( data_guojia['组织机构'][jj],'')

    data_guojiatianqi = pd.read_excel(filename, sheet_name = '国家天气站333',header=0,skiprows=0,index_col=None,)
                        
    for jj in range(data_guojiatianqi.shape[0]):
        data_guojiatianqi['站名'][jj] = data_guojiatianqi['站名'][jj].replace('国家气象观测站','')
        data_guojiatianqi['站名'][jj] = data_guojiatianqi['站名'][jj].replace( data_guojiatianqi['组织机构'][jj],'')
        if data_guojiatianqi['站名'][jj].find('尔罗斯')>=0:
            data_guojiatianqi['站名'][jj] = data_guojiatianqi['站名'][jj].replace('尔罗斯','')

    data_quyuzhan = pd.read_excel(filename, sheet_name = '区域站1048',header=0,skiprows=0,index_col=None,)
    for jj in range(data_quyuzhan.shape[0]):
        data_quyuzhan['站名'][jj] = data_quyuzhan['站名'][jj].replace('气象观测站','')
        
        data_quyuzhan['站名'][jj] = data_quyuzhan['站名'][jj].replace(data_quyuzhan['组织机构'][jj],'')
        if data_quyuzhan['站名'][jj].find('尔罗斯')>=0:
            data_quyuzhan['站名'][jj] = data_quyuzhan['站名'][jj].replace('尔罗斯','')
        


    lat=[]
    lon=[]
    staname=[]
    for ng in range(len(data_guojia['经度'])):
        lon.append(data_guojia['经度'][ng])
        lat.append(data_guojia['纬度'][ng])
        staname.append(data_guojia['站名'][ng])
    
    for ng in range(len(data_guojiatianqi['经度'])):
        lon.append(data_guojiatianqi['经度'][ng])
        lat.append(data_guojiatianqi['纬度'][ng])
        staname.append(data_guojiatianqi['站名'][ng])
    
    # for ng in range(len(data_quyuzhan['经度'])):
    #     lon.append(data_quyuzhan['经度'][ng])
    #     lat.append(data_quyuzhan['纬度'][ng])
    #     staname.append(data_quyuzhan['站名'][ng])



    # radfilepath='/Users/wenjianzhu/Downloads/ZZHN'
    radfilepath = sourcepath
    params['radarfile_path'] = radfilepath
    params['radarfile_name'] = radfilename
    params['pic_path'] = outpath
    params['timestr'] = timestr
    params['slat'] = slat
    params['nlat'] = nlat
    params['wlon'] = wlon
    params['elon'] = elon
    params['fontfile'] = fontname
    params['gis_lats'] = lat
    params['gis_lons'] = lon
    params['gis_name'] = staname
    params['breplace'] = True
    params['bdraw_crs'] = True
    params['ref_colorfile'] = '../common/gr2_colors/default_BR_PUP2.pal'
    params['vel_colorfile'] = '../common/gr2_colors/default_BV_PUP2.pal'
    params['figsize_width'] = 4
    params['fontsize_gis'] = 5
    params['fontsize_colorbar'] = 5
    params['fontsize_title'] = 6
    params['mapcolor'] = [0/255,0/255,0/255]
    params['dpi'] = 800
    params['pic_format'] = 'jpg'
    params['bdraw_title_ppi'] = False

    # params={'radarfile_path':radfilepath,
    #         'radarfile_name':radfilename,
    #         'mosaicfile_path':'',
    #         'mosaicfile_name':'',
    #         'pic_path':outpath,
    #         'timestr':timestr,
    #         'aws_min_file_path':'',
    #         'aws_min_file_name':'',
    #         'aws_hour_file_path':'',
    #         'aws_hour_file_name':'',
    #         'gis_name':staname,
    #         'gis_lats':lat,
    #         'gis_lons':lon,
    #         'slat':slat,
    #         'nlat':nlat,
    #         'wlon':wlon,
    #         'elon':elon,
    #         'ref_colorfile':'../common/gr2_colors/default_BR_PUP2.pal',
    #         'vel_colorfile':'../common/gr2_colors/default_BV_PUP2.pal',
    #         'fontfile':fontname,
    #         'dpi':800,
    #         'pic_format':'png',
    #         'figsize_width':4,
    #         'fontsize_gis':5,
    #         'fontsize_colorbar':5,
    #         'fontsize_title':6,
    #         'mapcolor':[0/255,0/255,0/255],
    #         'breplace':True, #如果图片文件已存在，是否重新绘制
    #         'bdraw_crs':False
    # }

    _draw_radar_other = DRAW_RADAR_OTHER(params)

    _draw_radar_other.read_vol_data()

    # _draw_radar_other.draw_ref_alone(subdir='回波强度',tilt=0,thred=-5)
    _draw_radar_other.draw_ref_alone(subdir='回波强度',tilt=1,thred=-5) 


    # _draw_radar_other.draw_vel_alone(subdir='径向速度',tilt=0)
    # _draw_radar_other.draw_vel_alone(subdir='径向速度',tilt=1)
    # _draw_radar_other.draw_vel_alone(subdir='径向速度',tilt=2)

    # _draw_radar_other.draw_vel_pre()
    _draw_radar_other.draw_vel_wind_barb()
    # _draw_radar_other.draw_vel_wind_quiver()
    # _draw_radar_other.draw_ref_pre()
    # _draw_radar_other.draw_ref_pre_wind_barb()

    # _draw_radar_other.get_cref_from_radar([_draw_radar_other.g_rad_lat,_draw_radar_other.g_rad_lon])
    # _draw_radar_other.get_cref_from_mosaicfile()
    # _draw_radar_other.get_cref_from_radar([35.6,114.0])
    # _draw_radar_other.draw_cref_pre()
    # _draw_radar_other.draw_cref_wind_barb()
    # _draw_radar_other.draw_cref_wind_quiver()
    # _draw_radar_other.draw_cref_pre_wind_barb()


# %%
import os
if __name__ == '__main__':
    pass
    freeze_support()
    # Pool不支持跨CPU的虚拟服务器，会出现页面不足的错误提示

    # 下面是针对不同的case进行绘图
    paramfilepath = '/Users/wenjianzhu/Downloads/雷达数据-xxx/绘图参数/回波强度'
    # paramfilepath = '/Users/wenjianzhu/Downloads/雷达数据-xxx/绘图参数/径向速度'
    # drawinfo = pd.read_csv(paramfilepath + os.sep + 'radardrawlist_20120612.csv',encoding='gb18030')
    # drawinfo = pd.read_csv(paramfilepath + os.sep + 'radardrawlist_20120701.csv',encoding='gb18030')
    # drawinfo = pd.read_csv(paramfilepath + os.sep + 'radardrawlist_20150608.csv',encoding='gb18030')
    # drawinfo = pd.read_csv(paramfilepath + os.sep + 'radardrawlist_20170905.csv',encoding='gb18030')
    drawinfo = pd.read_csv(paramfilepath + os.sep + 'radardrawlist_20210909.csv',encoding='gb18030')
    # drawinfo = pd.read_csv(paramfilepath + os.sep + 'radardrawlist_20190602.csv',encoding='gb18030')

    params = []
    nums=drawinfo.shape[0]
    # nums=1
    for nn in range(nums):
        pass
        curparam=ini_params()
        curparam['sourcepath'] = drawinfo['sourcepath'].iloc[nn]
        curparam['outpath'] = drawinfo['outpath'].iloc[nn]
        curparam['radfilename'] = drawinfo['filename'].iloc[nn]
        curparam['slat'] = drawinfo['start_lat'].iloc[nn]
        curparam['nlat'] = drawinfo['end_lat'].iloc[nn]
        curparam['wlon'] = drawinfo['start_lon'].iloc[nn]
        curparam['elon'] = drawinfo['end_lon'].iloc[nn]
        # aws_min_delta_t_file_path
        params.append(curparam)

    # MAXP = int(cpu_count()*0.5)
    MAXP=1#nums
    pools = Pool(MAXP)

    pools.map(draw_all, params)
    pools.close()
    pools.join()



    


"""
metradar is a python package to read or retrieve radar data

"""

__author__ = "The R & D Center for Weather Forecasting Technology in NMC, CMA"


__all__ = [s for s in dir() if not s.startswith('_')]